@@ -11,7 +11,7 @@ use WWW::Mechanize::Link;
 use IO::Socket::INET;
 
 use vars qw($VERSION %link_spec @CARP_NOT);
-$VERSION= '0.14';
+$VERSION= '0.15';
 @CARP_NOT=qw(Selenium::Remote::Driver);
 
 =head1 NAME
@@ -124,10 +124,10 @@ sub build_command_line {
     $options->{ launch_ghostdir } ||= $ghostdir_default;
     $options->{ launch_arg } ||= [];
     $options->{ phantomjs_arg } ||= [];
-    
+
     # config.js defaults config.port to 8910
     # this is the proper way to overwrite it (not sure wtf the PhantomJS parameter does above)
-    if ($options->{port}) { 
+    if ($options->{port}) {
         push @{ $options->{ launch_arg }}, "--port=$options->{ port }";
     }  # PhantomJS version 1.9.7
 
@@ -151,25 +151,26 @@ sub build_command_line {
         shift @cmd; # remove pipe-open
         @cmd= "| " . join " ", @cmd;
     };
-    
+
     @cmd
 };
 
 sub new {
     my ($class, %options) = @_;
 
+    my $localhost = '127.0.0.1';
     unless ( defined $options{ port } ) {
         my $port = 8910;
         while (1) {
             $port++, next unless IO::Socket::INET->new(
-	        Listen    => 5,
-	        Proto     => 'tcp',
-	        Reuse     => 1,
-	        LocalPort => $port
-	    );
-	    last;
+                Listen    => 5,
+                Proto     => 'tcp',
+                Reuse     => 1,
+                LocalPort => $port
+            );
+            last;
         }
-    	$options{ port } = $port;
+        $options{ port } = $port;
     }
 
     if (! exists $options{ autodie }) { $options{ autodie } = 1 };
@@ -180,15 +181,15 @@ sub new {
 
     unless ($options{pid}) {
         my @cmd= $class->build_command_line( \%options );
-    	$options{ kill_pid } = 1;
-    	if( @cmd > 1 ) {
-    	    # We can do a proper pipe-open
+        $options{ kill_pid } = 1;
+        if( @cmd > 1 ) {
+            # We can do a proper pipe-open
             my $mode = shift @cmd;
             $options{ pid } = open $options{fh}, $mode, @cmd
                 or die "Couldn't launch [@cmd]: $! / $?";
         } else {
-    	    # We can't do a proper pipe-open, so do the single-arg open
-    	    # in the hope that everything has been set up properly
+            # We can't do a proper pipe-open, so do the single-arg open
+            # in the hope that everything has been set up properly
             $options{ pid } = open $options{fh}, $cmd[0]
                 or die "Couldn't launch [$cmd[0]]: $! / $?";
         };
@@ -198,30 +199,35 @@ sub new {
         while ( time < $wait ) {
             my $t = time;
             my $socket = IO::Socket::INET->new(
-                PeerHost => '127.0.0.1',
+                PeerHost => $localhost,
                 PeerPort => $options{ port },
                 Proto    => 'tcp',
             );
-            last if $socket;
+            if( $socket ) {
+                close $socket;
+                sleep 1;
+                last;
+            };
             sleep 1 if time - $t < 1;
         }
     }
 
     # Connect to it
     eval {
-    	$options{ driver } ||= Selenium::Remote::Driver->new(
-    	    'port' => $options{ port },
-    	    auto_close => 0,
-    	    error_handler => sub {
-    	        #warn ref$_[0];
-    	        #warn "<<@CARP_NOT>>";
-    	        #warn ((caller($_))[0,1,2])
-    	        #    for 1..4;
-    	        local @CARP_NOT = (@CARP_NOT, ref $_[0],'Try::Tiny');
-    	        # Reraise the error
-    	        croak $_[1]
-    	    },
-    	);
+        $options{ driver } ||= Selenium::Remote::Driver->new(
+            'port' => $options{ port },
+            remote_server_addr => $localhost,
+            auto_close => 0,
+            error_handler => sub {
+                #warn ref$_[0];
+                #warn "<<@CARP_NOT>>";
+                #warn ((caller($_))[0,1,2])
+                #    for 1..4;
+                local @CARP_NOT = (@CARP_NOT, ref $_[0],'Try::Tiny');
+                # Reraise the error
+                croak $_[1]
+            },
+        );
         # (Monkey)patch Selenium::Remote::Driver
         $options{ driver }->commands->get_cmds->{get}->{no_content_success}= 0;
     };
@@ -229,7 +235,7 @@ sub new {
     # if PhantomJS started, but so slow or unresponsive that SRD cannot connect to it,
     # kill it manually to avoid waiting for it indefinitely
     if ( $@ ) {
-    	kill 9, $options{ pid } if $options{ kill_pid };
+        kill 9, delete $options{ pid } if $options{ kill_pid };
         die $@;
     }
 
@@ -382,7 +388,7 @@ JS
 =head2 C<< $mech->confirm( 'Really do this?' [ => 1 ]) >>
 
 Records a confirmation (which is "1" or "ok" by default), to be used
-whenever javascript fires a confirm dialog. If the message is not found, 
+whenever javascript fires a confirm dialog. If the message is not found,
 the answer is "cancel".
 
 =cut
@@ -469,14 +475,22 @@ JS
 }
 
 sub DESTROY {
-    #warn "Destroying " . ref $_[0];
     my $pid= delete $_[0]->{pid};
-    eval { 
-        my $dr= delete $_[0]->{ driver }; $dr->quit; undef $dr;
+
+    # Purge the filehandle - we should've opened that to /dev/null anyway:
+    if( my $child_out = $_[0]->{ fh }) {
+        local $/;
+        1 while <$child_out>;
+    };
+
+    eval {
+        my $dr= delete $_[0]->{ driver };
+        $dr->quit;
+        undef $dr;
+    };
+    if( $pid ) {
+        kill 'SIGKILL' => $pid;
     };
-    #warn "Killing $pid";
-    kill 9 => $pid
-        if $pid;
     %{ $_[0] }= (); # clean out all other held references
 }
 
@@ -568,7 +582,7 @@ sub get {
     # We need to stringify $url so it can pass through JSON
     my $phantom_res= $self->driver->get( "$url" );
     $self->post_process;
-    
+
     $self->update_response( $phantom_res );
 };
 
@@ -2017,7 +2031,13 @@ sub active_form {
 
 =head2 C<< $mech->dump_forms( [$fh] ) >>
 
-Prints a dump of the forms on the current page to $fh. If $fh is not specified or is undef, it dumps to STDOUT.
+  open my $fh, '>', 'form-log.txt'
+      or die "Couldn't open logfile 'form-log.txt': $!";
+  $mech->dump_forms( $fh );
+
+Prints a dump of the forms on the current page to
+the filehandle C<$fh>. If C<$fh> is not specified or is undef, it dumps
+to C<STDOUT>.
 
 =cut
 
@@ -2035,7 +2055,7 @@ sub dump_forms {
             if($type= $f->get_attribute('type') || '' ) {
                 $type= " ($type)";
             };
-            
+
             print {$fh} "    [", $f->get_attribute('tagName'), $type, "] ", $f->get_attribute('name') || '<no name>', "\n";
         };
     }
@@ -2919,9 +2939,9 @@ sub report_js_errors
 {
     my ( $self, @errors ) = @_;
     @errors = map {
-    	$_->{message} . 
-	( @{$_->{trace}} ? " at $_->{trace}->[-1]->{file} line $_->{trace}->[-1]->{line}" : '') .
-	( @{$_->{trace}} && $_->{trace}->[-1]->{function} ? " in function $_->{trace}->[-1]->{function}" : '')
+        $_->{message} .
+    ( @{$_->{trace}} ? " at $_->{trace}->[-1]->{file} line $_->{trace}->[-1]->{line}" : '') .
+    ( @{$_->{trace}} && $_->{trace}->[-1]->{function} ? " in function $_->{trace}->[-1]->{function}" : '')
     } @errors;
     Carp::carp("javascript error: @errors") ;
 }
